import uuid
import threading
import SocketServer
#from wsgiref.simple_server import make_server
import wsgiref
import wsgiref.simple_server
import subprocess
import os
import time
import logging

logger = logging.getLogger("logfun")
logger.setLevel(logging.DEBUG)

h1 = logging.FileHandler("server.log")
#f = logging.Formatter("%(levelname)s %(asctime)s %(funcName)s %(lineno)d %(message)s")
f = logging.Formatter("%(levelname)s %(asctime)s %(levelname)s %(message)s")
h1.setFormatter(f)
h1.setLevel(logging.DEBUG)
logger.addHandler(h1)


idx = 0
sim_root = '/tmp'

simulations = {}

# http://python.6.n6.nabble.com/Non-evil-multithreaded-WSGI-server-td1451634.html

class MyWSGIServer(SocketServer.ThreadingMixIn, wsgiref.simple_server.WSGIServer):
     pass 

#class MyWSGIServer(wsgiref.simple_server.WSGIServer):
#     pass 

def make_server(
     host, port, app, server_class=MyWSGIServer,  
     handler_class=wsgiref.simple_server.WSGIRequestHandler):
     return wsgiref.simple_server.make_server(host, port, app,  
                                              server_class, handler_class) 



IDLE_TIME = 15*60

class Simulation(object):
    def __init__(self, name):
        self.name = name
        self.p = subprocess.Popen(['python','-u', 'sim_driver.py'],
                             stdin = subprocess.PIPE, stdout=subprocess.PIPE)
        logger.info('New process '+self.name+ ' %d'%self.p.pid)
        self.last_time = time.time()

    def get_info(self):
        return 'hello'

    def do_command(self, command):
        self.p.stdin.write(command+'\n')
        self.last_time = time.time()
        a = self.p.stdout.readline()
        self.last_time = time.time()
        return a
 
    def is_alive(self):
         self.p.poll()
         return self.p.returncode==None

    def is_idle(self):
        t = time.time()
        logger.info('is_idle '+self.name+' '+str(t-self.last_time))
        return (t - self.last_time) > IDLE_TIME
             
    def shutdown(self):
         logger.info('Process killed '+self.name+ ' %d'%self.p.pid)
         self.p.kill()

def handle_request(env, start_response):
    global idx
    method = env['REQUEST_METHOD']

    logger.info('Request : '+str(method)+' '+env['PATH_INFO'])

    if method == 'GET':
         start_response('200 OK', [('Content-Type', 'text/plain')])        
         return []

    if method == 'POST':
        rname = env['PATH_INFO']
        if rname=='/auxin/sims' or rname =='/auxin/sims/':
            # Make new simulation
            idx = uuid.uuid4()
            sim_name = '/auxin/sims/'+str(idx)
            logger.info('New simulation request ')
            simulations[sim_name]=Simulation(sim_name)
            start_response('201 Created', [('Content-Type', 'text/plain'),
                                           ('Location', sim_name)])        
            return []
        else:
            rname = env['PATH_INFO']
            if rname in simulations:
                if not simulations[rname].is_alive():
                     logger.info('Dead simulation requested ')
                     del simulations[rname]
                     start_response('410 Gone', [])        
                     return []
                try:
                    r_size = int(env.get('CONTENT_LENGTH', 0))
                except (ValueError):
                    request_body_size = 0 
                command = env['wsgi.input'].read(r_size)
                print 'command', rname, command
                logger.info('Command : '+env['PATH_INFO']+command[0:min(len(command),20)])
                response = simulations[rname].do_command(command)

                if response[:9]!='EXCEPTION':
                     start_response('200 OK', [('Content-Type', 'text/plain')]) 
                     return [response]
                else:
                     logger.info('Internal server error '+rname)
                     start_response('500 Internal Server Error', [('Content-Type', 'text/plain')]) 
                     return [response]
            else:
                 logger.info('Not found '+rname)
                 start_response('404 Not found', [])        
            return []

    start_response('405 Method Not Allowed', [('Allow:', 'GET, POST')])
    return []

def gc_simulations():
     logger.info('gc_simulations')
     while(True):
          sims = list(simulations.itervalues()) 
          for s in sims:
               if s.is_idle() and s.is_alive():
                    s.shutdown()
          time.sleep(15*60)


background_thread = threading.Thread(target=gc_simulations)
background_thread.start()
httpd = make_server('', 80, handle_request)
httpd.serve_forever()
background_thread.stop()

