

import xml.etree.ElementTree as ET
from openalea.celltissue import TissueDB
from db_utilities import get_mesh, get_graph, get_wall_decomp, get_tissue_maps
from db_geom import ordered_wids_pids

def get_xml(db, preserve_oa_ids=False):
    mesh = get_mesh(db)
    pos = db.get_property('position')
    cell_type = db.get_property('cell_type')
    root = ET.Element('cellsetdata')
    tissue_maps = get_tissue_maps(db)

    max_y = max(v[1] for v in pos.itervalues())
    min_y = min(v[1] for v in pos.itervalues())
    if preserve_oa_ids:
        wall_map = dict((wid, int(wid)) for wid in mesh.wisps(1))
        cell_map = dict((cid, int(cid)) for cid in mesh.wisps(2))
    else:
        wall_map=tissue_maps['wall']
        if 'old_cell_id' in db.properties():
            cell_map = db.get_property('old_cell_id')
        else:
            cell_map = tissue_maps['cell']
    cells = ET.SubElement(root, 'cells', count = str(len(cell_map)))
    for cid, cidx in cell_map.iteritems():
        cell = ET.SubElement(cells, 'cell')
        cell.set('id', str(cidx))
        cell.set('group', str(cell_type[cid]))
        cell.set('truncated', 'False')
        walls = ET.SubElement(cell, 'walls')
        for wid in mesh.borders(2, cid):
            w=ET.SubElement(walls, 'wall')
            w.set('id', str(wall_map[wid]))

    walls = ET.SubElement(root, 'walls', count = str(len(wall_map)))
    for wid, widx in wall_map.iteritems():
        wall = ET.SubElement(walls, 'wall')
        wall.set('id', str(widx))
        wall.set('edgewall', 'false')
        wall.set('group', '0')
        points = ET.SubElement(wall, 'points')
        for pid in mesh.borders(1, wid):
            pt = ET.SubElement(points, 'point', x=str(pos[pid][0]), y=str(max_y - pos[pid][1]))
#    ET.dump(root)
    groups = ET.SubElement(root, 'groups')
    cellgroups = ET.SubElement(groups, 'cellgroups')
    cfg = db.get_config('config')    
    cell_types = { 2: 'epidermis',
                   4: 'cortex',
                   3: 'endodermis',
                   5: 'stele',
                   6: 'LRC1',
                   7: 'LRC2',
                   8: 'LRC3',
                   17: 'QC',
                   10: 'initials',
                   11: 'S1',
                   12: 'S2',
                   13: 'S3',
                   14: 'S4',
                   15: 'S5' }

    for ctid, ctname in cell_types.iteritems():
        ET.SubElement(cellgroups, 'group', { 'id': str(ctid),
                                             'name': ctname} )
    return ET.ElementTree(root)
    
def write_xml(db, outfile):
    get_xml(db).write(outfile)

def get_xml_string(db, preserve_oa_ids=False):
    return ET.tostring(get_xml(db, preserve_oa_ids).getroot())
